@@ -11,7 +11,8 @@ Tardos, and V. Vazirani eds., Algorithmic Game Theory, 2007.
 """
 import numpy as np
 import scipy.spatial
-from numba import jit, guvectorize
+from numba import jit, guvectorize, types
+from numba.typed import Dict
 
 
 def vertex_enumeration(g, qhull_options=None):
@@ -41,6 +42,20 @@ def vertex_enumeration(g, qhull_options=None):
     list(tuple(ndarray(float, ndim=1)))
         List containing tuples of Nash equilibrium mixed actions.
 
+    Examples
+    --------
+    >>> from pprint import pprint
+    >>> np.set_printoptions(precision=4)  # Reduce the digits printed
+    >>> bimatrix = [[(3, 3), (3, 2)],
+    ...             [(2, 2), (5, 6)],
+    ...             [(0, 3), (6, 1)]]
+    >>> g = NormalFormGame(bimatrix)
+    >>> NEs = vertex_enumeration(g)
+    >>> pprint(NEs)
+    [(array([1., 0., 0.]), array([1., 0.])),
+     (array([0.    , 0.3333, 0.6667]), array([0.3333, 0.6667])),
+     (array([0.8, 0.2, 0. ]), array([0.6667, 0.3333]))]
+
     """
     return list(vertex_enumeration_gen(g, qhull_options=qhull_options))
 
@@ -63,6 +78,21 @@ def vertex_enumeration_gen(g, qhull_options=None):
     tuple(ndarray(float, ndim=1))
         Tuple of Nash equilibrium mixed actions.
 
+    Examples
+    --------
+    >>> np.set_printoptions(precision=4)  # Reduce the digits printed
+    >>> bimatrix = [[(3, 3), (3, 2)],
+    ...             [(2, 2), (5, 6)],
+    ...             [(0, 3), (6, 1)]]
+    >>> g = NormalFormGame(bimatrix)
+    >>> gen = vertex_enumeration_gen(g)
+    >>> for NE in gen:
+    ...     print(NE)
+    ...
+    (array([1., 0., 0.]), array([1., 0.]))
+    (array([0.    , 0.3333, 0.6667]), array([0.3333, 0.6667]))
+    (array([0.8, 0.2, 0. ]), array([0.6667, 0.3333]))
+
     """
     try:
         N = g.N
@@ -109,18 +139,26 @@ def _vertex_enumeration_gen(labelings_bits_tup, equations_tup, trans_recips):
     ZERO_LABELING0_BITS = (np.uint64(1) << np.uint64(m)) - np.uint64(1)
     COMPLETE_LABELING_BITS = (np.uint64(1) << np.uint64(m+n)) - np.uint64(1)
 
+    labelings_bits_dict1 = Dict.empty(key_type=types.uint64,
+                                      value_type=types.intp)
+    for j in range(num_vertices1):
+        labelings_bits_dict1[labelings_bits_tup[1][j]] = j
+
     for i in range(num_vertices0):
-        if labelings_bits_tup[0][i] == ZERO_LABELING0_BITS:
+        bits0 = labelings_bits_tup[0][i]
+        if bits0 == ZERO_LABELING0_BITS:
             continue
-        for j in range(num_vertices1):
-            xor = labelings_bits_tup[0][i] ^ labelings_bits_tup[1][j]
-            if xor == COMPLETE_LABELING_BITS:
-                yield _get_mixed_actions(
-                    labelings_bits_tup[0][i],
-                    (equations_tup[0][i], equations_tup[1][j]),
-                    trans_recips
-                )
-                break
+        complement0 = bits0 ^ COMPLETE_LABELING_BITS
+        try:
+            j = labelings_bits_dict1[complement0]
+        except Exception:
+            continue
+
+        yield _get_mixed_actions(
+            bits0,
+            (equations_tup[0][i], equations_tup[1][j]),
+            trans_recips
+        )
 
 
 class _BestResponsePolytope:
